clear
addpath('.../MATLAB/stan/cmdstan-2.9.0')
addpath('.../MATLAB/stan/MatlabProcessManager-master')
addpath('.../MATLAB/stan/MatlabStan-2.7.0.0')

% This program estimates a logit model with random coefficient on price
% using data in long-form (multiple rows for each choice situation)



%%%%%%%%%%%%%%%%%%%%%%%%
%     SIMULATE DATA
%%%%%%%%%%%%%%%%%%%%%%%%

nvisits = 1000; % number of choice situations
nprovs = 5; % total number of choices
nindiv = 500; % number of individuals (used for individual-specific coef)
visitid = sort(repmat((1:nvisits)',nprovs,1));

% Set paramters
param.price = random('norm', -1.5, 1,[nindiv,1]); % Coefficients on price (random coefficient)
param.dchars = [-1.5; 0.5]; % Coefficient on distance, etc (no random coefficient)
param.provid = [0; 45; 40; 35; 50]; % Alt-specific FEs (outside option is 0)
    
% Simulate provider FE (1st alternative is outside option)
provid = repmat((1:nprovs)',nvisits,1);
prov_fe = repmat(param.provid,nvisits,1);

rand_integer = repmat(randi([0 (nindiv-1)],nvisits,1),[1,nprovs])';
rand_integer=rand_integer(:);
individ=rand_integer +1;

% Simulate Xs (1st alternative is outside option)
price = random('norm', 10, 10,[nvisits*nprovs,1]);
price(provid==1) = 0;
dist = random('norm', 20, 10,[nvisits*nprovs,1]);
dist(provid==1) = 0;
x2 = random('norm', 5, 10,[nvisits*nprovs,1]);

% Simulate choice set and subset X
choice_set=binornd(1,0.7,nvisits*nprovs,1);

prov_fe=prov_fe(choice_set==1 | provid==1,:);
price=price(choice_set==1 | provid==1,:);
dist=dist(choice_set==1 | provid==1,:);
x2=x2(choice_set==1 | provid==1,:);

individ=individ(choice_set==1 | provid==1,:);

visitid =grp2idx(visitid(choice_set==1 | provid==1,:));
provid=grp2idx(provid(choice_set==1 | provid==1,:));

% Constants
nchoices = size(visitid,1);
nvisits = max(visitid);
nprovs = max(provid);

% Simulate utility
util = prov_fe + param.price(individ).*price + [dist, x2]*param.dchars + random('ev', 0, 1,[nchoices,1]);

% Find max utility (including outside option with 0 utility)
max_u = accumarray(visitid,util,[],@max);
max_u = max_u(visitid);
choice = (max_u==util);

disp('Tabulate choices:')
disp(crosstab(choice, provid))

% Make choice ID within each visit
prov_id2 = zeros(nchoices,1);
Nchoiceset = zeros(nvisits,1);
for g = 1:nvisits
    prov_id2(visitid==g) = grp2idx(provid(visitid==g));
    Nchoiceset(g) = max(prov_id2(visitid==g));
end
choice2=prov_id2(choice);
individ2=individ(choice==1);

data = struct('Nchoice',nchoices,'Nvisit',nvisits,'Nprov',nprovs,'Nindiv',nindiv, ...
    'visit',visitid,'prov',provid,'indiv',individ2, ...
    'price',price,'Nx',2,'x',[dist, x2],'Nchoiceset',Nchoiceset,'y',choice2);




% Compile and run MCMC
fit1 = stan('file','mlogit2.stan','data',data,'iter',250,'file_overwrite',true,'verbose',true);
addlistener(fit1,'exit',@exitHandler);
%fit1.block(); % Block command line until finished

% Stop
%fit1.stop();

print(fit1);
fit1.traceplot;

disp('xi:')
disp([mean(fit1.extract('permuted',true).xi)', std(fit1.extract('permuted',true).xi)'])
disp('beta:')
disp([mean(fit1.extract('permuted',true).beta)', std(fit1.extract('permuted',true).beta)'])
disp('alpha_mu:')
disp([mean(fit1.extract('permuted',true).alpha_mu)', std(fit1.extract('permuted',true).alpha_mu)'])
disp('alpha_s:')
disp([mean(fit1.extract('permuted',true).alpha_s)', std(fit1.extract('permuted',true).alpha_s)'])


% Run (without re-compiling)
fit2 = stan('fit',fit1,'data',data,'iter',1000,'chains',4);
print(fit2);
